Attribute VB_Name = "mHookPaste"
Option Explicit

'declare external procedures (api calls)
    Public Declare Function SetWindowLong Lib "user32" _
                                Alias "SetWindowLongA" _
                                (ByVal hWnd As Long, _
                                ByVal nIndex As Long, _
                                ByVal dwNewLong As Long) As Long
                                
    Public Declare Function CallWindowProc Lib "user32" _
                                Alias "CallWindowProcA" _
                                (ByVal lpPrevWndFunc As Long, _
                                ByVal hWnd As Long, _
                                ByVal Msg As Long, _
                                ByVal wParam As Long, _
                                ByVal lParam As Long) As Long
                                
'declare api constants
    Private Const WM_PASTE = &H302
    Private Const GWL_WNDPROC = (-4)

'declare local collection
    Private colHookedCtrls      As New Collection
        'using a collection to allow multiple textboxes
        'to be subclassed simulatenously.
    
'declare local variables
    Private OrigWndProc         As Long
        'this variable will hold the memory address of the original
        'window procedure for each subclassed textbox
'

Public Sub HookTextBox_Numeric(ByVal hwndTxt As Long)
    '-------------------------------------------------------------------
    ' Usage: sets up subclassing for the specified textbox
    '-------------------------------------------------------------------

    'return the memory address of the window procedure for the textbox to be subclassed,
    '  set the memory address to the custom window procedure for the textbox
    OrigWndProc = SetWindowLong(hwndTxt, GWL_WNDPROC, AddressOf WndProc_Numeric)
    
    'record the original window procedure address in collection
    colHookedCtrls.Add OrigWndProc, "hwnd" & hwndTxt
End Sub

Public Sub UnHookTextBox(ByVal hwndTxt As Long)
    '-------------------------------------------------------------------
    ' Usage: releases subclassing for the specified textbox
    '-------------------------------------------------------------------

    'retrieve the memory address of the original window procedure from the collection
    OrigWndProc = CLng(colHookedCtrls("hwnd" & hwndTxt))
    
    'sets the memory address of the textbox's window procedure back to its original
    SetWindowLong hwndTxt, GWL_WNDPROC, OrigWndProc
    
    'remove the entry from the collection
    colHookedCtrls.Remove ("hwnd" & hwndTxt)
End Sub

Private Function WndProc_Numeric(ByVal hWnd As Long, ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
    '-------------------------------------------------------------------
    ' Usage: handles windows messages for the specifed textboxes
    '-------------------------------------------------------------------

    'init
    Dim vStr As String
    
    'retrieve the memory address of the original window procedure from the collection
    OrigWndProc = CLng(colHookedCtrls("hwnd" & hWnd))
    
    'get windows message
    If uMsg = WM_PASTE Then     'PASTE MESSAGE:
        'get clipboard text
        vStr = Clipboard.GetText(vbCFText)
        
        'validate the clipboard text
        If IsNumeric(vStr) Then
            'if numeric, allow pasting:
            '  call the original window procedure for the textbox
            WndProc_Numeric = CallWindowProc(OrigWndProc, hWnd, uMsg, wParam, lParam)
        Else
            'if not numeric, cancel the message
            WndProc_Numeric = 0
            
            'if clipboard was not empty, display error message
            If Not vStr = "" Then
                MsgBox "This field only accepts numeric data." & vbCrLf _
                    & "However, the clipboard currently contains non-numeric data.", , "Cannot Paste Data ..."
            End If
        End If
    Else        'ALL OTHER WINDOWS MESSAGES:
        'call the original window procedure for the textbox
        WndProc_Numeric = CallWindowProc(OrigWndProc, hWnd, uMsg, wParam, lParam)
    End If
End Function


